﻿using System.Collections.Generic;
using System.Data;
using System.Text;
using OAI.Util;

namespace OAI.Server.Catalog {
	public class Header {
		public Header(IOAIHandler handler, IDataRecord reader) {
			var prefix = handler.GetIdentifyPrefix();
			Identifier = OAIUtil.XmlEncode(string.Format("oai:{0}{1}", prefix, reader.GetValue(0)));
			DateStamp = reader.GetDateTime(1).ToUniversalTime().ToString("yyyy-MM-dd");
			SetSpecs = null;
			Status = OAIUtil.ReplaceDbNull(reader, 2, null, null);
		}

		public Header() {}

		public string Identifier { get; set; }
		public string DateStamp { get; set; }
		public List<string> SetSpecs { get; set; }
		public string Status { get; set; }

		public override string ToString() {
			var sb = new StringBuilder();

			sb.Append("<header");

			if (Status != null) {
				sb.Append(" status=\"");
				sb.Append(Status);
				sb.Append("\"");
			}
			sb.Append(">");

			sb.Append("<identifier>");
			sb.Append(Identifier);
			sb.Append("</identifier>");

			sb.Append("<datestamp>");
			sb.Append(DateStamp);
			sb.Append("</datestamp>");

			if (SetSpecs != null) {
				foreach (var setSpec in SetSpecs) {
					sb.Append("<setSpec>");
					sb.Append(setSpec);
					sb.Append("</setSpec>");
				}
			}

			sb.Append("</header>");

			return sb.ToString();
		}
	}
}
