﻿using System;
using OAI.Server.Verb;

namespace OAI.Server.Catalog {
	public class ResumptionMap {
		public string From { get; private set; }
		public string Until { get; private set; }
		public string MetadataPrefix { get; private set; }
		public int? CompleteListSize { get; set; }
		public int? Cursor { get; set; }
		public string ResumptionToken {
			get {
				return string.Format("{0}:{1}:{2}:{3}:{4}", CompleteListSize.Value, Cursor.Value, MetadataPrefix, From, Until);
			}
		}

		public ResumptionMap(string resumptionToken) {
			var split = resumptionToken.Split(':');
			if (split.Length != 5)
				throw new BadResumptionTokenException();

			try {
				CompleteListSize = int.Parse(split[0]);
				Cursor = int.Parse(split[1]);
				MetadataPrefix = split[2];
				From = split[3];
				Until = split[4];
			}
			catch {
				throw new BadResumptionTokenException();
			}

			if (Cursor > CompleteListSize)
				throw new BadResumptionTokenException();
		}

		public ResumptionMap(int size, int cursor, string metadataPrefix, string from, string until) {
			Cursor = cursor;
			CompleteListSize = size;
			MetadataPrefix = metadataPrefix;
			From = from;
			Until = until;
		}
	}
}
