﻿using System;
using System.Collections.Generic;
using System.Text;

namespace OAI.Server.Verb {
	public class Identify : ServerVerb {
		private static readonly List<string> validParamNames = new List<string> { "verb" };

		public static string Construct(IOAIHandler oaiHandler) {
			var baseURL = oaiHandler.GetBaseURL();
			var query = oaiHandler.GetQuery();

			var sb = new StringBuilder();
			sb.Append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
			sb.Append("<OAI-PMH xmlns=\"http://www.openarchives.org/OAI/2.0/\"");
			sb.Append(" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
			sb.Append(" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/");
			sb.Append(" http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd\">");
			sb.Append("<responseDate>");
			sb.Append(CreateResponseDate(DateTime.Now));
			sb.Append("</responseDate>");

			sb.Append(GetRequestElement(query, validParamNames, baseURL));

			try {
				if (HasBadArguments(query, validParamNames, validParamNames)) {
					throw new BadArgumentException();
				}

				sb.Append("<Identify>");

				sb.Append("<repositoryName>");
				sb.Append(oaiHandler.GetRepositoryName());
				sb.Append("</repositoryName>");
				sb.Append("<baseURL>");
				sb.Append(baseURL);
				sb.Append("</baseURL>");
				sb.Append("<protocolVersion>2.0</protocolVersion>");

				var emails = oaiHandler.GetIdentifyAdminEmails();

				foreach (var email in emails) {
					sb.Append("<adminEmail>");
					sb.Append(email);
					sb.Append("</adminEmail>");
				}

				sb.Append("<earliestDatestamp>");
				sb.Append(oaiHandler.GetIdentifyEarliestDatestamp());
				sb.Append("</earliestDatestamp>");
				sb.Append("<deletedRecord>");
				sb.Append(oaiHandler.GetDeletedRecord());
				sb.Append("</deletedRecord>");

				var granularity = oaiHandler.GetGranularity();
				if (granularity != null) {
					sb.Append("<granularity>");
					sb.Append(granularity);
					sb.Append("</granularity>");
				}

				sb.Append("<compression>gzip</compression>");
				sb.Append("<compression>deflate</compression>");

				var customDescriptions = oaiHandler.GetCustomIdentifyDescriptions();

				foreach (var customDescription in customDescriptions) {
					sb.Append("<description>");
					sb.Append(customDescription);
					sb.Append("</description>");
				}

				sb.Append("</Identify>");
			} catch (BadArgumentException e) {
				sb.Append(e.Message);
			}

			sb.Append("</OAI-PMH>");

			return sb.ToString();
		}
	}
}
