﻿using System;
using System.Collections.Generic;
using System.Text;

namespace OAI.Server.Verb {
	public class ListMetadataFormats : ServerVerb {
		private static readonly List<string> validParamNames = new List<string> { "verb", "identifier" };
		private static readonly List<string> requiredParamNames = new List<string> { "verb" };

		public static string Construct(IOAIHandler oaiHandler) {
			var baseURL = oaiHandler.GetBaseURL();
			var query = oaiHandler.GetQuery();

			var sb = new StringBuilder();
			sb.Append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
			sb.Append("<OAI-PMH xmlns=\"http://www.openarchives.org/OAI/2.0/\"");
			sb.Append(" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
			sb.Append(" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/");
			sb.Append(" http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd\">");
			sb.Append("<responseDate>");
			sb.Append(CreateResponseDate(DateTime.Now));
			sb.Append("</responseDate>");

			sb.Append(GetRequestElement(query, validParamNames, baseURL));

			try {
				if (HasBadArguments(query, requiredParamNames, validParamNames)) {
					throw new BadArgumentException();
				}

				var formats = oaiHandler.GetAllMetadataFormats();

				sb.Append("<ListMetadataFormats>");
				foreach (var format in formats) {
					sb.Append("<metadataFormat>");
					sb.Append("<metadataPrefix>");
					sb.Append(format.Label);
					sb.Append("</metadataPrefix>");
					sb.Append("<schema>");
					sb.Append(format.SchemaURL);
					sb.Append("</schema>");
					sb.Append("<metadataNamespace>");
					sb.Append(format.NamespaceURI);
					sb.Append("</metadataNamespace>");
					sb.Append("</metadataFormat>");
				}
				sb.Append("</ListMetadataFormats>");
			} catch (BadArgumentException e) {
				sb.Append(e.Message);
			} catch (NoMetadataFormatsException e) {
				sb.Append(e.Message);
			}

			sb.Append("</OAI-PMH>");
			return sb.ToString();
		}
	}
}
