﻿using System;
using System.Collections.Generic;
using System.Text;

namespace OAI.Server.Verb {
	public class ListSets : ServerVerb {
		private static readonly List<string> validParamNames = new List<string> { "verb", "resumptionToken" };
		private static readonly List<string> requiredParamNames = new List<string> { "verb" };

		public static string Construct(IOAIHandler oaiHandler) {
			var baseURL = oaiHandler.GetBaseURL();
			var query = oaiHandler.GetQuery();

			var oldResumptionToken = query["resumptionToken"];

			var sb = new StringBuilder();
			sb.Append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
			sb.Append("<OAI-PMH xmlns=\"http://www.openarchives.org/OAI/2.0/\"");
			sb.Append(" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
			sb.Append(" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/");
			sb.Append(" http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd\">");
			sb.Append("<responseDate>");
			sb.Append(CreateResponseDate(DateTime.Now));
			sb.Append("</responseDate>");

			sb.Append(GetRequestElement(query, validParamNames, baseURL));

			try {
				var sets = oldResumptionToken == null ? oaiHandler.ListSets() : oaiHandler.ListSets(oldResumptionToken);

				if (sets != null) {
					if (HasBadArguments(query, requiredParamNames, validParamNames)) {
						throw new BadArgumentException();
					}

					sb.Append("<ListSets>");
					if (sets.Sets != null) {
						foreach (var set in sets.Sets) {
							sb.Append(set);
						}
					}

					var newResumptionMap = sets.ResumptionMap;

					if (newResumptionMap != null) {
						var newResumptionToken = newResumptionMap.ResumptionToken;
						var completeListSize = newResumptionMap.CompleteListSize;
						var cursor = newResumptionMap.Cursor;

						sb.Append("<resumptionToken");

						if (completeListSize != null) {
							sb.Append(" completeListSize=\"");
							sb.Append(completeListSize.Value);
							sb.Append("\"");
						}

						if (cursor != null) {
							sb.Append(" cursor=\"");
							sb.Append(cursor.Value);
							sb.Append("\"");
						}

						sb.Append(">");
						sb.Append(newResumptionToken);
						sb.Append("</resumptionToken>");
					} else if (oldResumptionToken != null) {
						sb.Append("<resumptionToken />");
					}

					sb.Append("</ListSets>");
				}
			} catch (NoSetHierarchyException e) {
				sb.Append(e.Message);
			} catch (BadResumptionTokenException e) {
				sb.Append(e.Message);
			}

			sb.Append("</OAI-PMH>");
			return sb.ToString();
		}
	}
}
