﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Text;
using OAI.Util;

namespace OAI.Server.Verb {
	public abstract class ServerVerb {
		protected string _xmlText;

		protected ServerVerb() {
		}

		protected void Init(string xmlText) {
#if DEBUG
			Debug.WriteLine("ServerVerb.Init: xmlText=" + xmlText);
#endif
			_xmlText = xmlText;
		}

		protected ServerVerb(string xmlText) {
			Init(xmlText);
			StatusCode = 200;
		}

		public int StatusCode { get; set; }

		public string Message { get; private set; }

		protected void SetError(int statusCode, string message) {
			StatusCode = statusCode;
			Message = message;
		}

		public static string CreateResponseDate(DateTime date) {
			var utcDate = date.ToUniversalTime();
			var sb = new StringBuilder();
			sb.AppendFormat(
				"{0,4:0000}-{1,2:00}-{2,2:00}T{3,2:00}:{4,2:00}:{5,2:00}Z",
				utcDate.Year,
				utcDate.Month,
				utcDate.Day,
				utcDate.Hour,
				utcDate.Minute,
				utcDate.Second
			);

			return sb.ToString();
		}

		public static string GetRequestElement(NameValueCollection query, List<string> validParamNames, string baseURL) {
			return GetRequestElement(query, validParamNames, baseURL, false);
		}

		protected static string GetRequestElement(NameValueCollection query, List<string> validParamNames, string baseURL, bool xmlEncodeSetSpec) {
			var sb = new StringBuilder();
			sb.Append("<request");

			foreach (string name in query.Keys) {
				if (!validParamNames.Contains(name)) continue;

				var value = query[name];
				if (string.IsNullOrEmpty(value)) continue;

				sb.Append(" ");
				sb.Append(name);
				sb.Append("=\"");

				if (!xmlEncodeSetSpec && "set".Equals(name)) {
					sb.Append(value);
				} else {
					sb.Append(OAIUtil.XmlEncode(value));
				}

				sb.Append("\"");
			}

			sb.Append(">");
			sb.Append(OAIUtil.XmlEncode(baseURL));
			sb.Append("</request>");

			return sb.ToString();
		}

		protected static bool HasBadArguments(NameValueCollection query, List<string> requiredParamNames, List<string> validParamNames) {
			foreach (var name in requiredParamNames) {
				var value = query[name];
				if (string.IsNullOrEmpty(value))
					return false;
			}

			foreach (string name in query.Keys) {
				if (!validParamNames.Contains(name) && query[name].Length > 0)
					return true;
			}

			var identifier = query["identifier"];

			try {
				if (!string.IsNullOrEmpty(identifier)) {
					new Uri(identifier);
				}
			} catch (UriFormatException) {
				return true;
			}

			return false;
		}

		public override string ToString() {
			return _xmlText;
		}

		public static Dictionary<string, Type> Verbs {
			get {
				var serverVerbsMap = new Dictionary<string, Type>();
				serverVerbsMap["ListRecords"] = typeof(ListRecords);
				serverVerbsMap["ListIdentifiers"] = typeof(ListIdentifiers);
				serverVerbsMap["GetRecord"] = typeof(GetRecord);
				serverVerbsMap["Identify"] = typeof(Identify);
				serverVerbsMap["ListMetadataFormats"] = typeof(ListMetadataFormats);
				serverVerbsMap["ListSets"] = typeof(ListSets);
				serverVerbsMap["Reload"] = typeof(Reload);
				return serverVerbsMap;
			}
		}
	}
}