﻿using System;
using System.Data;
using System.IO;
using System.Text;
using System.Xml;

namespace OAI.Util {
	public class OAIUtil {
/*
		private static XmlDocument _staticDoc;
		private static StringWriter _staticStringWriter;
		private static XmlWriter _staticXmlWriter;
*/

		public static string ReplaceDbNull(IDataRecord reader, int col, string repl, string type) {
			if (reader.IsDBNull(col))
				return repl;

			switch (type)
			{
				case "float":
				case "double":
					return reader.GetDouble(col).ToString().Replace(",", ".");
				case "date":
					return reader.GetDateTime(col).ToString("yyyy-MM-dd");
				default:
					var value = reader.GetValue(col).ToString();

					return XmlEncode(value.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;"));
			}
		}

		public static string GetFinestFrom(string from, string granularity) {
			if (string.IsNullOrEmpty(from)) {
				from = "0000-01-01";
			}

			if (from.Length > granularity.Length) {
				from = null;
			} else if (from.Length < granularity.Length) {
				switch (granularity) {
					case "YYYY-MM-DD":
						from = from + "0000-01-01".Substring(from.Length);
						break;
					case "YYYY-MM-DDT00:00:00Z":
						from = from + "0000-01-01T:00:00:00Z".Substring(from.Length);
						break;
				}
			}

			try
			{
				DateTime.Parse(from);
			}
			catch
			{
				from = null;
			}

			return from;
		}

		public static string GetFinestUntil(string until, string granularity) {
			if (string.IsNullOrEmpty(until)) {
				until = "9999-12-31";
			}

			if (until.Length > granularity.Length) {
				until = null;
			} else if (until.Length < granularity.Length) {
				switch (granularity) {
					case "YYYY-MM-DD":
						until = until + "9999-12-31".Substring(until.Length);
						break;
					case "YYYY-MM-DDY00:00:00Z":
						until = until + "9999-12-31T:23:59:59Z".Substring(until.Length);
						break;
				}
			}

			try {
				DateTime.Parse(until);
			} catch {
				until = null;
			}

			return until;
		}

		public static string XmlEncode(string str) {

			if (str == null) return "";

			/*
			if (_staticDoc == null) {
				_staticDoc = new XmlDocument();
				_staticDoc.LoadXml("<text></text>");
				_staticStringWriter = new StringWriter();
				var settings = new XmlWriterSettings { ConformanceLevel = ConformanceLevel.Fragment };
				_staticXmlWriter = XmlWriter.Create(_staticStringWriter, settings);
			}
			lock (_staticDoc) {
				_staticDoc.LastChild.InnerText = str;
				str = _staticDoc.LastChild.InnerXml;
			}*/

			// ASCII enforcement
			var sb = new StringBuilder();
			var chars = str.ToCharArray();
			for (var i = 0; i < chars.Length; i++) {
				var c = chars[i];
				// goes beyond ASCII charset
				/*
				if (c > 255) {
					lock (_staticStringWriter) {
						lock (_staticXmlWriter) {
							_staticXmlWriter.WriteCharEntity(c);
							_staticXmlWriter.Flush();
							var _sb = _staticStringWriter.GetStringBuilder();
							sb.Append(_sb.ToString());
							_sb.Length = 0;
						}
					}
				} else*/ if (c > 31 || c == 9 || c == 10 || c == 13) {
					sb.Append(c);
				}
			}
			return sb.ToString();
		}
	}
}
